package explorviz.live_trace_processing.connector;

import java.io.IOException;
import java.net.InetSocketAddress;
import java.net.MalformedURLException;
import java.net.URL;
import java.nio.ByteBuffer;
import java.nio.channels.SocketChannel;

import explorviz.live_trace_processing.configuration.Configuration;
import explorviz.live_trace_processing.filter.AbstractSink;
import explorviz.live_trace_processing.record.IRecord;
import explorviz.live_trace_processing.record.misc.HostApplicationMetaDataRecord;
import explorviz.live_trace_processing.writer.IWriter;

public class TCPConnector extends AbstractSink implements IWriter {
	private URL providerURL;

	private SocketChannel socketChannel;

	private final Configuration configuration;

	public TCPConnector(final String hostname, final int port, final Configuration configuration) {
		this.configuration = configuration;
		try {
			setProviderURL(new URL("http://" + hostname + ":" + port));
		} catch (final MalformedURLException e) {
			e.printStackTrace();
		}
	}

	protected TCPConnector(final Configuration configuration) {
		this.configuration = configuration;
		providerURL = null;
	}

	@Override
	public URL getProviderURL() {
		return providerURL;
	}

	@Override
	public void setProviderURL(final URL providerURL) {
		this.providerURL = providerURL;
	}

	public void init() {
		try {
			connect();
		} catch (final IOException e) {
			e.printStackTrace();
		}
	}

	@Override
	public void connect() throws IOException {
		socketChannel = SocketChannel.open(new InetSocketAddress(getProviderURL().getHost(),
				getProviderURL().getPort()));

		if (socketChannel.isConnected()) {
			// socketChannel.write(StringRegistry.getAllStringRegistryRecords());
		}
	}

	@Override
	protected void processRecord(final IRecord record,
			final HostApplicationMetaDataRecord hostApplicationMetaData) {
		// send(); TODO
	}

	private void send(final ByteBuffer buffer) {
		if (socketChannel.isConnected()) {
			try {
				while (buffer.hasRemaining()) {
					socketChannel.write(buffer);
				}
			} catch (final IOException e) {
				System.out.println("WARNING: Connection was closed");
				// TODO reconnect
				disconnect();
			}
		}
	}

	@Override
	public final void disconnect() {
		if (socketChannel.isConnected()) {
			try {
				socketChannel.close();
			} catch (final IOException e) {
				e.printStackTrace();
			}
		}
	}

	public void cleanup() {
		disconnect();
	}
}
