package explorviz.live_trace_processing.filter.reconstruction;

import java.util.ArrayList;
import java.util.List;

import explorviz.live_trace_processing.record.event.AbstractOperationEventRecord;
import explorviz.live_trace_processing.record.event.normal.AfterFailedOperationEventRecord;
import explorviz.live_trace_processing.record.event.normal.AfterOperationEventRecord;
import explorviz.live_trace_processing.record.event.normal.BeforeOperationEventRecord;
import explorviz.live_trace_processing.record.misc.HostApplicationMetaDataRecord;
import explorviz.live_trace_processing.record.trace.Trace;

class TraceReconstructionBuffer {
	// private static final Comparator<AbstractOperationEvent> COMPARATOR = new
	// AbstractOperationEventComperator();

	private static final int INITIAL_EVENT_CAPACITY = 100;

	private HostApplicationMetaDataRecord traceMetadata;
	private final List<AbstractOperationEventRecord> events = new ArrayList<AbstractOperationEventRecord>(
			INITIAL_EVENT_CAPACITY);

	private boolean closeable;
	private boolean damaged;

	private int openEvents;

	private long maxLoggingTimestamp = -1;
	private int maxOrderIndex = -1;

	public final void insertEvent(final AbstractOperationEventRecord event) {
		setMaxLoggingTimestamp(event);
		final int orderIndex = setMaxOrderIndex(event);

		if (event instanceof BeforeOperationEventRecord) {
			if (orderIndex == 0) {
				closeable = true;
			}
			openEvents++;
		} else if (event instanceof AfterOperationEventRecord) {
			openEvents--;
		} else if (event instanceof AfterFailedOperationEventRecord) {
			openEvents--;
		}

		if (!events.add(event)) { // TODO
			System.out.println("Duplicate entry for orderIndex " + orderIndex + " with traceId "
					+ event.getTraceId());
			damaged = true;
		}
	}

	public final long getMaxLoggingTimestamp() {
		return maxLoggingTimestamp;
	}

	private final void setMaxLoggingTimestamp(final AbstractOperationEventRecord event) {
		final long loggingTimestamp = event.getLoggingTimestamp();
		if (loggingTimestamp > maxLoggingTimestamp) {
			maxLoggingTimestamp = loggingTimestamp;
		}
	}

	private final int setMaxOrderIndex(final AbstractOperationEventRecord event) {
		final int orderIndex = event.getOrderIndex();
		if (orderIndex > maxOrderIndex) {
			maxOrderIndex = orderIndex;
		}
		return orderIndex;
	}

	public void setTrace(final HostApplicationMetaDataRecord trace) {
		if (traceMetadata != null) {
			damaged = true;
			return;
		}
		traceMetadata = trace;
	}

	public final boolean isFinished() {
		return !isInvalid();
	}

	public final boolean isInvalid() {
		return ((openEvents != 0) || events.isEmpty() || ((maxOrderIndex + 1) != events.size())
				|| damaged || !closeable);
	}

	public final Trace toTrace() {
		return new Trace(traceMetadata, events, events.get(0).getTraceId());
	}

	// private static final class AbstractOperationEventComperator implements
	// Comparator<AbstractOperationEvent> {
	// @Override
	// public int compare(final AbstractOperationEvent o1, final
	// AbstractOperationEvent o2) {
	// return o1.getOrderIndex() - o2.getOrderIndex();
	// }
	// }
}
